"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const DiffDom = require("diff-dom");
var EventType;
(function (EventType) {
    EventType["AjaxContentRequested"] = "AjaxContentRequested";
    EventType["AjaxContentRejected"] = "AjaxContentRejected";
    EventType["AjaxContentWillLoad"] = "AjaxContentWillLoad";
    EventType["AjaxContentLoaded"] = "AjaxContentLoaded";
})(EventType = exports.EventType || (exports.EventType = {}));
const defaultDiffDomOpts = {
    preDiffApply(info) {
        if (info.diff.action === 'removeElement' &&
            info.node.tagName === 'SCRIPT') {
            return true;
        }
    },
};
const defaultStartOpts = {
    elemSelector: 'a[href]:not([target]):not([download])',
    observeMutations: true,
    smoothScrollAnchors: false,
    diffDomOpts: defaultDiffDomOpts,
};
const defaultNavOpts = {
    smoothScrollAnchor: undefined,
    pushHistory: true,
};
const domParser = new DOMParser();
let diffDom;
let mutationObserver;
let options = defaultStartOpts;
let started = false;
function start(partStartOpts) {
    const startOpts = Object.assign({}, defaultStartOpts, partStartOpts);
    if (started) {
        stop();
    }
    diffDom = new DiffDom(Object.assign({}, defaultDiffDomOpts, startOpts.diffDomOpts));
    document.querySelectorAll(startOpts.elemSelector)
        .forEach(bindElem);
    addEventListener('popstate', handleHistoryPopState);
    if (startOpts.observeMutations) {
        mutationObserver = new MutationObserver(handleMutations);
        mutationObserver.observe(document.documentElement, {
            attributes: true,
            childList: true,
            subtree: true,
        });
    }
    started = true;
    options = startOpts;
}
exports.start = start;
function stop() {
    if (!started) {
        return;
    }
    started = false;
    if (mutationObserver) {
        mutationObserver.disconnect();
        mutationObserver = null;
    }
    for (const elem of document.getElementsByTagName('*')) {
        if (elem instanceof HTMLElement) {
            unbindElem(elem);
        }
    }
    removeEventListener('popstate', handleHistoryPopState);
    options = defaultStartOpts;
    diffDom = null;
}
exports.stop = stop;
function navigate(url, partNavOpts) {
    return __awaiter(this, void 0, void 0, function* () {
        const navOpts = Object.assign({}, defaultNavOpts, partNavOpts);
        const eventCancelled = !document.dispatchEvent(new CustomEvent(EventType.AjaxContentRequested, {
            cancelable: true,
            detail: { url },
        }));
        const response = yield fetch(url);
        if (eventCancelled) {
            return;
        }
        if (!response.ok) {
            document.dispatchEvent(new CustomEvent(EventType.AjaxContentRejected, {
                detail: { response },
            }));
            return;
        }
        const contentType = response.headers.get('content-type') || '';
        if (contentType.indexOf('text/html') === undefined) {
            location.href = url;
            return;
        }
        const responseText = yield response.text();
        const newDoc = domParser.parseFromString(responseText, 'text/html');
        document.dispatchEvent(new CustomEvent(EventType.AjaxContentWillLoad, {
            detail: { response, document: newDoc },
        }));
        const headDiff = diffDom.diff(document.head, newDoc.head);
        const bodyDiff = diffDom.diff(document.body, newDoc.body);
        diffDom.apply(document.head, headDiff);
        diffDom.apply(document.body, bodyDiff);
        if (navOpts.pushHistory) {
            history.pushState(null, '', url);
        }
        if (location.hash) {
            const scrollToElem = document.querySelector(location.hash);
            if (scrollToElem) {
                const smoothScroll = (navOpts.smoothScrollAnchor === undefined ?
                    options.smoothScrollAnchors : navOpts.smoothScrollAnchor);
                if (!smoothScroll) {
                    scrollToElem.scrollIntoView();
                }
                else {
                    scrollToElem.scrollIntoView({ behavior: 'smooth' });
                }
            }
        }
        document.querySelectorAll(options.elemSelector)
            .forEach(bindElem);
        document.dispatchEvent(new CustomEvent(EventType.AjaxContentLoaded, {
            detail: { response },
        }));
    });
}
exports.navigate = navigate;
function handleMutations(mutations) {
    for (const mutation of mutations) {
        if (mutation.type === 'attributes') {
            handleAttributesMutation(mutation);
        }
        else if (mutation.type === 'childList') {
            handleChildListMutation(mutation);
        }
    }
}
function handleAttributesMutation(mutation) {
    if (mutation.target instanceof HTMLAnchorElement &&
        mutation.target.matches(options.elemSelector) &&
        mutation.attributeName === 'href') {
        unbindElem(mutation.target);
        bindElem(mutation.target);
    }
}
function handleChildListMutation(mutation) {
    for (const addedNode of mutation.addedNodes) {
        if (!(addedNode instanceof HTMLElement)) {
            continue;
        }
        if (addedNode instanceof HTMLAnchorElement &&
            addedNode.matches(options.elemSelector)) {
            bindElem(addedNode);
        }
        const elems = addedNode.querySelectorAll(options.elemSelector);
        for (const elem of elems) {
            if (elem instanceof HTMLAnchorElement) {
                bindElem(elem);
            }
        }
    }
}
function bindElem(elem) {
    if (!(elem instanceof HTMLAnchorElement)) {
        throw new Error(`Only ${HTMLAnchorElement.name} is supported.`);
    }
    if (elem.href && elem.host === location.host) {
        elem.removeEventListener('click', handleElemClick);
        elem.addEventListener('click', handleElemClick);
    }
}
function unbindElem(elem) {
    elem.removeEventListener('click', handleElemClick);
}
function handleElemClick(event) {
    event.preventDefault();
    if (!this.href || this.href === location.href) {
        return;
    }
    navigate(this.href, { pushHistory: true });
}
function handleHistoryPopState() {
    navigate(location.href, { pushHistory: false });
}
//# sourceMappingURL=index.js.map